/***************************************************************************
                           client application class
                             -------------------
    begin                : Tue Dec 06 21:54 CET 2005
    copyright            : (C) 2005 by Stefan Gehn
    email                : metz \AT\ gehn \DOT\ net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lcapplication.h"

#ifdef Q_WS_MAC
#include <Carbon/Carbon.h>
#endif

#ifdef Q_WS_X11
//#include <time.h>
//#include <sys/time.h>
#include <qx11info_x11.h>
#include <qdesktopwidget.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/SM/SMlib.h>

// Atoms required for monitoring the freedesktop.org notification area
static Atom manager_atom = 0;
static Atom tray_selection_atom = 0;
Window root_window = 0;
Window tray_owner = None;

//static Atom kde_net_wm_user_time = 0;
//Time qt_x_last_input_time = CurrentTime;

#ifdef KeyPress
#ifndef FIXX11H_KeyPress
#define FIXX11H_KeyPress
const int XKeyPress = KeyPress;
#undef KeyPress
const int KeyPress = XKeyPress;
#endif  // FIXX11H_KeyPress
#undef KeyPress
#endif // KeyPress

#endif // Q_WS_X11



#ifdef Q_WS_X11
void setTrayOwnerWindow(Display *dsp)
{
	// ignore events from the old tray owner
	if (tray_owner != None)
		XSelectInput(dsp, tray_owner, 0);

	// obtain the Window handle for the new tray owner
	XGrabServer(dsp);
	tray_owner = XGetSelectionOwner(dsp, tray_selection_atom);

	// we have to be able to spot DestroyNotify messages on the tray owner
	if (tray_owner != None)
		XSelectInput(dsp, tray_owner, StructureNotifyMask|PropertyChangeMask);
	XUngrabServer(dsp);
	XFlush(dsp);
}
#endif


LCApplication::LCApplication(int argc, char **argv) : QApplication(argc, argv)
{
	qDebug("LCApplication::LCApplication()");
#ifdef Q_WS_X11
	const int max = 20;
	Atom* atoms[max];
	char* names[max];
	Atom atoms_return[max];
	int n = 0;

	//atoms[n] = &kde_net_wm_user_time;
	//names[n++] = (char *) "_NET_WM_USER_TIME";
	atoms[n] = &manager_atom;
	names[n++] = (char *) "MANAGER";

	Display *dsp = QX11Info::display();

	XInternAtoms( dsp, names, n, false, atoms_return );

	for (int i = 0; i < n; i++ )
		*atoms[i] = atoms_return[i];

	// get the selection type we'll use to locate the notification tray
	char buf[32];
	snprintf(buf, sizeof(buf), "_NET_SYSTEM_TRAY_S%d",
		XScreenNumberOfScreen( XDefaultScreenOfDisplay(dsp) ));
	tray_selection_atom = XInternAtom(dsp, buf, false);

	// make a note of the window handle for the root window
	root_window = QApplication::desktop()->winId();

	XWindowAttributes attr;

	// this is actually futile, since Qt overrides it at some
	// unknown point in the near future.
	XGetWindowAttributes(dsp, root_window, &attr);
	XSelectInput(dsp, root_window, attr.your_event_mask | StructureNotifyMask);

	setTrayOwnerWindow(dsp);
#endif
}


#ifdef Q_WS_X11
bool LCApplication::x11EventFilter( XEvent *_event )
{
	switch ( _event->type )
	{
	case ClientMessage:
		if (_event->xclient.window == root_window && _event->xclient.message_type == manager_atom)
		{
			// A new notification area application has
			// announced its presence
			setTrayOwnerWindow(_event->xclient.display);
			emit newTrayOwner();
		}
		break;

	case DestroyNotify:
		if (_event->xdestroywindow.event == tray_owner)
		{
			// there is now no known notification area.
			// We're still looking out for the MANAGER
			// message sent to the root window, at which
			// point we'll have another look to see
			// whether a notification area is available.
			tray_owner = 0;
			emit trayOwnerDied();
		}
		break;
	}

	// process the event normally
	return false;
}
#endif


#ifdef Q_WS_MAC
bool LCApplication::macEventFilter( EventHandlerCallRef, EventRef inEvent )
{
	UInt32 eclass = GetEventClass(inEvent);
	int etype = GetEventKind(inEvent);

	if(eclass == 'eppc' && etype == kEventAppleEvent)
		emit dockActivated();

	return false;
}
#endif

