/***************************************************************************
                            main window contents
                             -------------------
    begin                : Thu Dec 01 22:13 CET 2005
    copyright            : (C) 2005 by Stefan Gehn
    email                : metz \AT\ gehn \DOT\ net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include "lcmainwidget.h"
#include "ui_lcmwidget.h"
#include "lcpio/misc.h"
#include "lcpio/lcpline.h"

#include <qpainter.h>
#include <qlist.h>

QPixmap renderIcon(const QString& baseIcon, const QString &overlayIcon)
{
	QPixmap base(baseIcon);
	QPixmap overlay(overlayIcon);
	if (!overlay.isNull()) // QPainter is allergic to null pixmaps!
	{
		QPainter p(&base);
		int x = base.width() - overlay.width();
		if (x < 0) x = 0;
		int y = base.height() - overlay.height();
		if (y < 0) y = 0;
		p.drawPixmap(x, y, overlay);
	}
	return base;
}


LCMainWidget::LCMainWidget(QWidget *parent) : QWidget(parent)
{
	//qDebug("LCMainWidget::LCMainWidget()");
	mWidget = new Ui::LCMWidget();
	mWidget->setupUi(this);
   updateClientStatus(CLT_DISCON);
}


void LCMainWidget::updateLineInfo(t_lcp3_info_tput lineInfo,
	double /*clientCost*/, double /*lineCost*/)
{
	//qDebug("LCMainWidget::updateLineInfo()");
	//qDebug("%d of %d used on download", lineInfo.dns/1024, mWidget->pbDown->maximum());
	mWidget->pbDown->setValue(QMIN((int)(lineInfo.dns/1024), mWidget->pbDown->maximum()));
	mWidget->lbDownSpeed->setText(tr("%1/s").arg(LCPIO::formatSize(lineInfo.dns)));
	mWidget->lbDownTotal->setText(LCPIO::formatSize(lineInfo.dnt * 1024));

	//qDebug("%d of %d used on upload", lineInfo.ups/1024, mWidget->pbUp->maximum());
	mWidget->pbUp->setValue(QMIN((int)(lineInfo.ups/1024), mWidget->pbUp->maximum()));
	mWidget->lbUpSpeed->setText(tr("%1/s").arg(LCPIO::formatSize(lineInfo.ups)));
	mWidget->lbUpTotal->setText(LCPIO::formatSize(lineInfo.upt * 1024));

	mWidget->lbClientTimeVal->setText(LCPIO::formatTime(lineInfo.client));
	mWidget->lbLineTimeVal->setText(LCPIO::formatTime(lineInfo.line));
}


void LCMainWidget::updateSelectedLine(LcpLine *line)
{
	if (!line)
	   return;
	//qDebug("LCMainWidget::updateSelectedLine()");
   mWidget->pbDown->setRange(0, line->maxThroughputDown() / (1024*8));
   mWidget->pbUp->setRange(0, line->maxThroughputUp() / (1024*8));
}


void LCMainWidget::updateClientStatus(int clientStatus)
{
	//qDebug("LCMainWidget::updateClientStatus()");
	mWidget->lbClientStateVal->setToolTip(LCPIO::clientStatusToString(clientStatus));
   switch(clientStatus)
   {
    case CLT_DISCON:
    case CLT_NOAUTH:
		mWidget->lbClientStateVal->setPixmap(QPixmap(":/pixmaps/16x16/offline.png"));
      resetView();
      break;
    case CLT_OFFLINE:
		mWidget->lbClientStateVal->setPixmap(QPixmap(":/pixmaps/16x16/offline.png"));
		mWidget->lbClientTimeVal->setText(LCPIO::formatTime(0));
      break;
    case CLT_DIALING:
		mWidget->lbClientStateVal->setPixmap(QPixmap(":/pixmaps/16x16/dialing.png"));
      break;
    case CLT_ONLINE:
		mWidget->lbClientStateVal->setPixmap(QPixmap(":/pixmaps/16x16/online.png"));
      break;
    case CLT_CLOSING:
		mWidget->lbClientStateVal->setPixmap(QPixmap(":/pixmaps/16x16/dialing.png"));
      break;
    case CLT_ABORTING:
		mWidget->lbClientStateVal->setPixmap(QPixmap(":/pixmaps/16x16/dialing.png"));
	}
}


void LCMainWidget::updateLineStatus(int lineStatus)
{
  	//qDebug("LCMainWidget::updateLineStatus()");
	mWidget->lbLineStateVal->setToolTip(LCPIO::lineStatusToString(lineStatus));
	switch(lineStatus)
	{
	case CST_DOWN:
		mWidget->lbLineStateVal->setPixmap(QPixmap(":/pixmaps/16x16/offline.png"));
		break;
	case CST_DIALING:
	case CST_CLOSING:
		mWidget->lbLineStateVal->setPixmap(QPixmap(":/pixmaps/16x16/dialing.png"));
		break;
	case CST_UP_SERVER:
	case CST_UP_USER:
		mWidget->lbLineStateVal->setPixmap(QPixmap(":/pixmaps/16x16/online.png"));
		break;
	case CST_CLOSE_ERROR:
		mWidget->lbLineStateVal->setPixmap(QPixmap(":/pixmaps/16x16/stop.png"));
		break;
	case CST_LOCKED_UP_CLT:
		mWidget->lbLineStateVal->setPixmap(renderIcon(":/pixmaps/16x16/online.png",
			":/pixmaps/overlay/lock_overlay.png"));
		break;
	case CST_LOCKED_DOWN:
		mWidget->lbLineStateVal->setPixmap(renderIcon(":/pixmaps/16x16/offline.png",
			":/pixmaps/overlay/lock_overlay.png"));
		break;
	case CST_LOCKED_UP_TIMED:
		mWidget->lbLineStateVal->setPixmap(renderIcon(":/pixmaps/16x16/online.png",
			":/pixmaps/overlay/lock_overlay.png"));
		break;
	}
}


void LCMainWidget::resetView()
{
	//qDebug("LCMainWidget::resetView()");
  	LcpLine l;
	l.setMaxThroughputUp(1000000);
	l.setMaxThroughputDown(1000000);
	updateSelectedLine(&l);

   updateLineStatus(CST_DOWN);

   t_lcp3_info_tput tmp;
   tmp.client = tmp.line = 0;
   tmp.dnl = tmp.upl = tmp.dns = tmp.ups = tmp.dnt = tmp.upt= 0;
   updateLineInfo(tmp, 0, 0);
}
