/***************************************************************************
                             client main window
                             -------------------
    begin                : Thu Dec 01 16:26 CET 2005
    copyright            : (C) 2005 by Stefan Gehn
    email                : metz \AT\ gehn \DOT\ net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lcmainwindow.h"
#include "lcpio/lcpio.h"
#include "lcpio/lcpline.h"
#include "lcpio/lcp3.h"
#include "lcsettings.h"
#include "lcmainwidget.h"
#include "trayicon.h"
#include "lcsettingsdlg.h"
#include "lcminidisplay.h"

#include <qaction.h>
#include <qactiongroup.h>
#include <qapplication.h>
#include <qmenu.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qtoolbar.h>
#include <qtimer.h>
//#include <qstatusbar.h>
#include <qinputdialog.h>

LCMainWindow::LCMainWindow() : QMainWindow()
{
	qDebug("LCMainWindow::LCMainWindow()");

	setObjectName("LCMainWindow");
	setWindowTitle(tr("QLineControl"));

	mTray = 0;
	mTrayMenu = 0;
	mIO = new LcpIO(this);
	mMainWidget = new LCMainWidget(this);

	setCentralWidget(mMainWidget);
	setMaximumHeight(mMainWidget->minimumSizeHint().height());

	mSettingsDlg = new LCSettingsDlg(this);

	connect(qApp, SIGNAL(dockActivated()),
		this, SLOT(slotTrayClosed())); // Used for OSX

	// connect mainwindow to lcpio
	connect(mIO, SIGNAL(allLinesReceived(QList<LcpLine *> *)),
		this, SLOT(slotAllLinesReceived(QList<LcpLine *> *)));

	connect(mIO, SIGNAL(selectedLineChanged(LcpLine *)),
		this, SLOT(slotSelectedLineChanged(LcpLine *)));

	connect(mIO, SIGNAL(clientstatusChanged(int)),
		this, SLOT(slotClientStatus(int)));

	connect(mIO, SIGNAL(selectedLineStatusChanged(int)),
		this, SLOT(slotLineStatus(int)));

	connect(mIO, SIGNAL(badAuth()), this, SLOT(slotAuthFailed()));


	// connect mainwidget (window contents) to lcpio
	/*connect(mIO, SIGNAL(allLinesReceived(QList<LcpLine *> *)),
		mMainWidget, SLOT(updateLinesList(QList<LcpLine *> *)));*/
	connect(mIO, SIGNAL(clientstatusChanged(int)),
		mMainWidget, SLOT(updateClientStatus(int)));
	connect(mIO, SIGNAL(selectedLineChanged(LcpLine *)),
		mMainWidget, SLOT(updateSelectedLine(LcpLine *)));
	connect(mIO, SIGNAL(selectedLineStatusChanged(int)),
		mMainWidget, SLOT(updateLineStatus(int)));
	connect(mIO, SIGNAL(selectedLineInfoUpdate(t_lcp3_info_tput, double, double)),
		mMainWidget, SLOT(updateLineInfo(t_lcp3_info_tput, double, double)));

	setupActions();
	setupMenubar();
	setupToolbar();
	setupStatusbar();

	// set initial gui status
	slotClientStatus(CLT_DISCON);
	slotLineStatus(CST_DOWN);

	// set up systray, autoonline, redial etc.
	slotLoadSettings();

	connect(LCSettings::self(), SIGNAL(settingsChanged()),
		this, SLOT(slotLoadSettings()));

	mMiniDisp = new LCMiniDisplay();
	connect(mIO, SIGNAL(clientstatusChanged(int)),
		mMiniDisp, SLOT(updateClientStatus(int)));
	connect(mIO, SIGNAL(selectedLineChanged(LcpLine *)),
		mMiniDisp, SLOT(updateSelectedLine(LcpLine *)));
	connect(mIO, SIGNAL(selectedLineStatusChanged(int)),
		mMiniDisp, SLOT(updateLineStatus(int)));
	connect(mIO, SIGNAL(selectedLineInfoUpdate(t_lcp3_info_tput, double, double)),
		mMiniDisp, SLOT(updateLineInfo(t_lcp3_info_tput, double, double)));

	QTimer::singleShot(0, this, SLOT(slotInit()));
}


LCMainWindow::~LCMainWindow()
{
	qDebug("LCMainWindow::~LCMainWindow()");

	if (LCSettings::self()->getRememberWindowState())
		LCSettings::self()->saveWindowSettings(this);

	delete mMiniDisp;
	mMiniDisp = 0;

	if (mTray)
	{
		delete mTray;
		mTray = 0;
	}

	delete mIO;
	mIO = 0;
}


void LCMainWindow::setupActions()
{
	mQuitAction = new QAction(QIcon(":/pixmaps/16x16/exit.png"),
		tr("&Quit", "File|Quit"), this);
	mQuitAction->setShortcut(tr("Ctrl+Q", "File|Quit"));

	mHideShowAction = new QAction("", this);
	slotPrepareTrayMenu(); // sets label accordingly

	// Server related actions
	mConnectAction = new QAction(QIcon(":/pixmaps/16x16/connect.png"),
		tr("&Connect", "File|Connect"), this);
	mConnectAction->setShortcut(tr("Ctrl+Up", "File|Connect"));
	mDisconnectAction = new QAction(QIcon(":/pixmaps/16x16/disconnect.png"),
		tr("&Disconnect", "File|Disconnect"), this);
	mDisconnectAction->setShortcut(tr("Ctrl+Down", "File|Disconnect"));

	// Line related actions
	mOnlineAction = new QAction(QIcon(":/pixmaps/16x16/online.png"),
		tr("&Online", "Line|Online"), this);
	mOnlineAction->setShortcut(tr("Alt+Up", "Line|Online"));
	mOfflineAction = new QAction(QIcon(":/pixmaps/16x16/offline.png"),
		tr("O&ffline", "Line|Offline"), this);
	mOfflineAction->setShortcut(tr("Alt+Down", "Line|Offline"));
	mLockAction = new QAction(QIcon(":/pixmaps/16x16/lock.png"),
		tr("&Lock", "Line|Lock"), this);
	mLockAction->setShortcut(tr("Ctrl+L", "Line|Lock"));

	mLineSelectorAction = new QActionGroup(this);
	mLineSelectorAction->setExclusive(true);

	mCfgAction = new QAction(QIcon(":/pixmaps/16x16/configure.png"),
		tr("&Preferences...", "Settings|Preferences"), this);

	connect(mQuitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
	connect(mHideShowAction, SIGNAL(triggered()), this, SLOT(toggleVisibility()));
	connect(mConnectAction, SIGNAL(triggered()), this, SLOT(slotConnect()));
	connect(mDisconnectAction, SIGNAL(triggered()), mIO, SLOT(disconnectFromServer()));
	connect(mOnlineAction, SIGNAL(triggered()), mIO, SLOT(online()));
	connect(mOfflineAction, SIGNAL(triggered()), mIO, SLOT(offline()));
	connect(mLockAction, SIGNAL(triggered()), mIO, SLOT(toggleLineLock()));
	connect(mCfgAction, SIGNAL(triggered()), mSettingsDlg, SLOT(exec()));
}


void LCMainWindow::setupMenubar()
{
	QMenu *m;
	m = menuBar()->addMenu(tr("&Client"));
	m->addAction(mConnectAction);
	m->addAction(mDisconnectAction);
	m->addSeparator();
	m->addAction(mQuitAction);

	m = menuBar()->addMenu(tr("&Line"));
	mLinesMenu = m->addMenu(tr("Lines"));
	m->addSeparator();
	m->addAction(mOnlineAction);
	m->addAction(mOfflineAction);
	m->addSeparator();
	m->addAction(mLockAction);

	mSettingsMenu = menuBar()->addMenu(tr("Se&ttings"));
	mSettingsMenu->addAction(mCfgAction);
	mSettingsMenu->addSeparator();
	// more added later

	menuBar()->addSeparator();

	QMenu *menuHelp = menuBar()->addMenu(tr("&Help"));
	menuHelp->addAction(tr("&About"), this, SLOT(slotAbout()));
	menuHelp->addAction(tr("About &Qt"), this, SLOT(slotAboutQt()));
}


void LCMainWindow::setupToolbar()
{
	QToolBar *tb = addToolBar(tr("Main Toolbar"));
	tb->setObjectName("MainToolbar");
	tb->setIconSize(QSize(16, 16));
	mSettingsMenu->addAction(tb->toggleViewAction());

	tb->addAction(mConnectAction);
	tb->addAction(mDisconnectAction);
	tb->addSeparator();
	tb->addAction(mOnlineAction);
	tb->addAction(mOfflineAction);
}


void LCMainWindow::setupStatusbar()
{
	// TODO
}


void LCMainWindow::setupTray()
{
	bool use = LCSettings::self()->getUseTray();

	if (!mTray && use)
	{
		mTrayMenu = new QMenu(this);
		mTrayMenu->addAction(mHideShowAction);
		mTrayMenu->addSeparator();
		mTrayMenu->addAction(mConnectAction);
		mTrayMenu->addAction(mDisconnectAction);
		mTrayMenu->addSeparator();
		mTrayMenu->addAction(mOnlineAction);
		mTrayMenu->addAction(mOfflineAction);
		mTrayMenu->addSeparator();
		mTrayMenu->addAction(mCfgAction);
		mTrayMenu->addAction(mQuitAction);

		connect(mTrayMenu, SIGNAL(aboutToShow()), this, SLOT(slotPrepareTrayMenu()));

		mTray = new TrayIcon(this);
		mTray->setWMDock(false);
		mTray->setPopup(mTrayMenu);
		updateTrayIcon();
		connect(mTray, SIGNAL(clicked(const QPoint &, int)), SLOT(toggleVisibility()));
		connect(mTray, SIGNAL(closed()), SLOT(slotTrayClosed()));
		connect(qApp, SIGNAL(newTrayOwner()), mTray, SLOT(newTrayOwner())); // Used on X11
		connect(qApp, SIGNAL(trayOwnerDied()), mTray, SLOT(hide())); // Used on X11

		mTray->show();
	}
	else if (mTray && !use)
	{
		delete mTray;
		delete mTrayMenu;
		mTray = 0;
		mTrayMenu = 0;
	}
}


void LCMainWindow::slotInit()
{
	//qDebug("LCMainWindow::slotInit()");

	if (LCSettings::self()->getRememberWindowState())
		LCSettings::self()->loadWindowSettings(this);
	else
		show();

	// failsafe
	if (!mTray && isHidden())
		show();

	if (LCSettings::self()->getAutoConnect())
		slotConnect();
}


void LCMainWindow::slotPrepareTrayMenu()
{
	mHideShowAction->setText(
		isHidden() ? tr("R&estore", "Traymenu|Show") : tr("Hid&e", "Traymenu|Hide"));
}


void LCMainWindow::slotAllLinesReceived(QList<LcpLine *> *lines)
{
	//qDebug("LCMainWidget::updateLinesList()");
	QList<QAction *> actions = mLineSelectorAction->actions();
	while (!actions.isEmpty())
		delete actions.takeFirst();

	if (lines)
	{
		QAction *a = 0;
		foreach(LcpLine *l, *lines)
		{
			if (l && l->lineId() > 0)
			{
				a = new QAction(l->name(), mLineSelectorAction);
				a->setData(l->lineId());
				a->setCheckable(true);
			}
		}
		mLinesMenu->addActions(mLineSelectorAction->actions());
	}
}


void LCMainWindow::slotSelectedLineChanged(LcpLine *line)
{
	if (!line)
		return;
	//qDebug("LCMainWidget::slotSelectedLineChanged()");
	int id = line->lineId();
	QList<QAction *> actions = mLineSelectorAction->actions();
	foreach(QAction *a, actions)
	{
		if (a->data() == id)
		{
			a->setChecked(true);
			break;
		}
	}
}


void LCMainWindow::slotClientStatus(int clientStatus)
{
	//qDebug("LCMainWindow::slotClientStatus()");
	switch(clientStatus)
	{
	case CLT_DISCON:
		mConnectAction->setEnabled(true);
		mDisconnectAction->setEnabled(false);
		mOnlineAction->setEnabled(false);
		mOfflineAction->setEnabled(false);
		mLinesMenu->setEnabled(false);
		//statusBar()->showMessage(tr("Disconnected"));
		break;

	case CLT_NOAUTH:
		mConnectAction->setEnabled(false);
		mDisconnectAction->setEnabled(false);
		mOnlineAction->setEnabled(false);
		mOfflineAction->setEnabled(false);
		mLinesMenu->setEnabled(false);
		break;

	case CLT_OFFLINE:
		mConnectAction->setEnabled(false);
		mDisconnectAction->setEnabled(true);
		mOnlineAction->setEnabled(true);
		mOfflineAction->setEnabled(false);
		mLinesMenu->setEnabled(true);
		//LCSettings *s = LCSettings::self();
		//statusBar()->showMessage(tr("Connected to %1:%2").arg(s->getServerHost()).arg(s->getServerPort()));
		break;

	case CLT_DIALING:
		mConnectAction->setEnabled(false);
		mDisconnectAction->setEnabled(false);
		mOnlineAction->setEnabled(false);
		mOfflineAction->setEnabled(true); // allow aborting an dial attempt
		mLinesMenu->setEnabled(false);
		break;

	case CLT_ONLINE:
		mConnectAction->setEnabled(false);
		// forbid disconnecting while client is online
		mDisconnectAction->setEnabled(false);
		mOnlineAction->setEnabled(false);
		mOfflineAction->setEnabled(true);
		mLinesMenu->setEnabled(true);
		break;

	// client cannot do anything in these states!
	case CLT_ABORTING:
	case CLT_CLOSING:
		mConnectAction->setEnabled(false);
		mDisconnectAction->setEnabled(false);
		mOnlineAction->setEnabled(false);
		mOfflineAction->setEnabled(false);
		mLinesMenu->setEnabled(false);
		break;
	}

	mCStatus = clientStatus;
	updateTrayIcon();
}


void LCMainWindow::slotLineStatus(int lineStatus)
{
	//qDebug("LCMainWindow::slotLineStatus()");
	switch(lineStatus)
	{
	case CST_DOWN:
	case CST_UP_USER:
	case CST_DIALING:
	case CST_CLOSING:
	case CST_CLOSE_ERROR:
	case CST_LOCKED_UP_TIMED:
	case CST_LOCKED_DOWN:
		mLockAction->setEnabled(false);
		break;
	case CST_UP_SERVER:
		mLockAction->setText(tr("&Lock"));
		mLockAction->setIcon(QIcon(":/pixmaps/16x16/lock.png"));
		mLockAction->setEnabled(true);
		break;
	case CST_LOCKED_UP_CLT:
		mLockAction->setText(tr("Un&lock"));
		mLockAction->setIcon(QIcon(":/pixmaps/16x16/unlock.png"));
		mLockAction->setEnabled(true);
		break;
	}
	mLStatus = lineStatus;
	updateTrayIcon();
}


void LCMainWindow::slotAuthFailed()
{
	QMessageBox::warning(this, tr("QLineControl - Authentication failed"),
		tr("Could not authenticate with server."), QMessageBox::Ok, 0);
}


void LCMainWindow::slotConnect()
{
	QString user, pass;
	LCSettings *s = LCSettings::self();

	if (s->getServerHost().isEmpty())
	{
		if (QMessageBox::question(this, tr("QLineControl - No server configured"),
			tr("No server-hostname has been configured in the settings.\n" \
			"Would you like to do that now?"), QMessageBox::Yes, QMessageBox::No)
			== QMessageBox::Yes)
		{
			mSettingsDlg->exec();
			if (s->getServerHost().isEmpty())
				return;
		}
		else
		{
			return;
		}
	}

	if (s->getUseAuth())
	{
		user = s->getAuthName();

		if (s->getStoreAuthPass())
			pass = s->getAuthPass();
		else
		{
			pass = QInputDialog::getText(this, tr("QLineControl - Password"),
				tr("Password:"), QLineEdit::Password);
		}

		if (pass.isNull())
		{
			//statusBar()->showMessage(tr("Connecting aborted, no password provided by user"));
			return;
		}
	}

	//statusBar()->showMessage(tr("Connecting to %1:%2...").arg(s->getServerHost()).arg(s->getServerPort()));
	mIO->connectToServer(s->getServerHost(), s->getServerPort(), user, pass);
}


void LCMainWindow::slotLoadSettings()
{
	qDebug("LCMainWindow::slotLoadSettings() loading settings from config");
	setupTray();
	//mIO->setAutoReconnect(use, delay, retries);
	//mIO->setAutoRedial(use, delay, retries);
	mIO->setAutoOnline(LCSettings::self()->getAutoOnline());
}


void LCMainWindow::slotTrayClosed()
{
	//qDebug("LCMainWindow::slotTrayClosed()");
	if(isHidden())
		show();
}


void LCMainWindow::toggleVisibility()
{
	//qDebug("LCMainWindow::toggleVisibility()");
	if(isHidden())
	{
		show();
		if(isMinimized())
		{
			if(isMaximized())
				showMaximized();
			else
				showNormal();
		}
		raise();
		activateWindow();
	}
	else
	{
		hide();
	}
}


void LCMainWindow::closeEvent(QCloseEvent *e)
{
	//qDebug("LCMainWindow::closeEvent()");
#ifdef Q_WS_MAC

	hide();
	e->accept();

#else

	e->accept();
	if (!mTray)
	{
		//qDebug("LCMainWindow::closeEvent(), no tray icon, quitting...");
		qApp->quit();
	}

#endif
}


void LCMainWindow::updateTrayIcon()
{
	QString base, overlay;

	// Windows has a 16x16 tray, X11 has a 22x22 tray
#ifdef WIN32
	QString pixPath = ":/pixmaps/16x16/";
#else
	QString pixPath = ":/pixmaps/22x22/";
#endif

	if (!mTray)
	   return;
	//qDebug("LCMainWindow::updateTrayIcon()");
	switch(mCStatus)
	{
	case CLT_DISCON:
	case CLT_NOAUTH:
	case CLT_CLOSING:
	case CLT_OFFLINE:
		base = pixPath + "offline.png";
		break;
	case CLT_ONLINE:
		base = pixPath + "online.png";
		break;
	case CLT_DIALING:
	case CLT_ABORTING:
		base = pixPath + "dialing.png";
		break;
	}

	switch(mLStatus)
	{
	case CST_CLOSE_ERROR:
		base = pixPath + "stop.png";
		overlay = "";
		break;
	case CST_DOWN:
	case CST_CLOSING:
	case CST_LOCKED_DOWN:
		overlay = ":/pixmaps/overlay/red_overlay";
		break;
	case CST_DIALING:
		overlay = ":/pixmaps/overlay/yellow_overlay";
		break;
	case CST_UP_USER:
		overlay = ":/pixmaps/overlay/blue_overlay";
		break;
	case CST_UP_SERVER:
	case CST_LOCKED_UP_CLT:
	case CST_LOCKED_UP_TIMED:
		overlay = ":/pixmaps/overlay/green_overlay";
		break;
	}

	QPixmap basePm(base);
	QPixmap ovPm(overlay);
	if (!ovPm.isNull()) // QPainter is allergic to null pixmaps!
	{
		QPainter p(&basePm);
		//int x = basePm.width() - ovPm.width();
		//if (x < 0) x = 0;
		int y = basePm.height() - ovPm.height();
		if (y < 0) y = 0;
		p.drawPixmap(0, y, ovPm);
	}

	mTray->setToolTip(tr("Client: %1\nLine: %2")
		.arg(LCPIO::clientStatusToString(mCStatus), LCPIO::lineStatusToString(mLStatus)));
	mTray->setIcon(basePm);
}


void LCMainWindow::slotAbout()
{
	QMessageBox::about(this, tr("QLineControl - About"),
		tr("<qt align=center><b>QLineControl 0.1</b><br><br>" \
			"A simple LineControl client<br>" \
			"<br>" \
			"(c) 2005 Stefan Gehn <metz@gehn.net></qt>"));
}


void LCMainWindow::slotAboutQt()
{
	QMessageBox::aboutQt(this, tr("QLineControl - About Qt"));
}

