/***************************************************************************
                            main window contents
                             -------------------
    begin                : Thu Dec 01 22:13 CET 2005
    copyright            : (C) 2005 by Stefan Gehn
    email                : metz \AT\ gehn \DOT\ net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lcminidisplay.h"
#include "lcpio/misc.h"
#include "lcpio/lcpline.h"
#include "lcsettings.h"

#include <qboxlayout.h>
#include <qprogressbar.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qevent.h>
#include <qtimer.h>

LCMiniDisplay::LCMiniDisplay()
	: QFrame(0, Qt::Tool | Qt::X11BypassWindowManagerHint | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint)
{
	//qDebug("LCMiniDisplay::LCMiniDisplay()");
	bMove = false;

	setFrameStyle(QFrame::Panel | QFrame::Raised);
	setLineWidth(1);

	setupUi();

	updateClientStatus(CLT_DISCON);
	updateLineStatus(CST_DOWN);

	QPoint pos = LCSettings::self()->getMiniDispPos();
	if (!pos.isNull())
		move(pos);

	loadSettings();
	connect(LCSettings::self(), SIGNAL(settingsChanged()), this, SLOT(loadSettings()));
}


LCMiniDisplay::~LCMiniDisplay()
{
	//qDebug("LCMiniDisplay::~LCMiniDisplay()");
	LCSettings::self()->setMiniDispPos(pos());
}


void LCMiniDisplay::mousePressEvent( QMouseEvent * e )
{
	if (!bMove && e->button() == Qt::LeftButton && e->modifiers() & Qt::AltModifier)
	{
	   bMove = true;
	   mMoveOrigin = e->pos();
	}
}


void LCMiniDisplay::mouseMoveEvent( QMouseEvent * e )
{
	if (bMove)
		move(e->globalPos() - mMoveOrigin);
}


void LCMiniDisplay::mouseReleaseEvent ( QMouseEvent * e )
{
	if (e->button() == Qt::LeftButton)
		bMove = false;
}


void LCMiniDisplay::setupUi()
{
	QVBoxLayout *innerLayout = 0;
	QHBoxLayout *hl = new QHBoxLayout(this);
	hl->setMargin(4);
	hl->setSpacing(4);

	{
		innerLayout = new QVBoxLayout();
		innerLayout->setMargin(0);
		innerLayout->setSpacing(1);

		pbDown = new QProgressBar(this);
		pbDown->setTextVisible(false);
		pbDown->setMinimumSize(24, 12); // Warning: height must be a multiple of 4

		lbDownSpeed = new QLabel(this);
		lbDownSpeed->setAlignment(Qt::AlignRight | Qt::AlignVCenter);

		lbDownTotal = new QLabel(this);
		lbDownTotal->setAlignment(Qt::AlignRight | Qt::AlignVCenter);

		innerLayout->addWidget(lbDownSpeed);
		innerLayout->addWidget(pbDown);
		innerLayout->addWidget(lbDownTotal);
	}
	hl->addLayout(innerLayout);

	{
		innerLayout = new QVBoxLayout();
		innerLayout->setMargin(0);
		innerLayout->setSpacing(1);

		pbUp = new QProgressBar(this);
		pbUp->setMinimumSize(24, 12); // Warning: height must be a multiple of 4
		pbUp->setTextVisible(false);

		lbUpSpeed = new QLabel(this);
		lbUpSpeed->setAlignment(Qt::AlignRight | Qt::AlignVCenter);

		lbUpTotal = new QLabel(this);
		lbUpTotal->setAlignment(Qt::AlignRight | Qt::AlignVCenter);

		innerLayout->addWidget(lbUpSpeed);
		innerLayout->addWidget(pbUp);
		innerLayout->addWidget(lbUpTotal);
	}
	hl->addLayout(innerLayout);

	{
		innerLayout = new QVBoxLayout();
		innerLayout->setMargin(0);
		innerLayout->setSpacing(0);

		lbLineTime = new QLabel(this);
		innerLayout->addWidget(lbLineTime);
	}
	hl->addLayout(innerLayout);
}


void LCMiniDisplay::updateLineInfo(t_lcp3_info_tput lineInfo,
	double /*clientCost*/, double /*lineCost*/)
{
	//qDebug("LCMiniDisplay::updateLineInfo()");
	//qDebug("%d of %d used on download", lineInfo.dns/1024, mWidget->pbDown->maximum());
	pbDown->setValue(QMIN((int)(lineInfo.dns/1024), pbDown->maximum()));
	lbDownSpeed->setText(QString("%1/s").arg(LCPIO::formatSize(lineInfo.dns)));
	lbDownTotal->setText(LCPIO::formatSize(lineInfo.dnt * 1024));

	//qDebug("%d of %d used on upload", lineInfo.ups/1024, mWidget->pbUp->maximum());
	pbUp->setValue(QMIN((int)(lineInfo.ups/1024), pbUp->maximum()));
	lbUpSpeed->setText(QString("%1/s").arg(LCPIO::formatSize(lineInfo.ups)));
	lbUpTotal->setText(LCPIO::formatSize(lineInfo.upt * 1024));

	//lbClientTimeVal->setText(LCPIO::formatTime(lineInfo.client));
	lbLineTime->setText(LCPIO::formatTime(lineInfo.line));
}


void LCMiniDisplay::updateSelectedLine(LcpLine *line)
{
	if (!line)
		return;
	//qDebug("LCMiniDisplay::updateSelectedLine()");
	pbDown->setRange(0, line->maxThroughputDown() / (1024*8));
	pbUp->setRange(0, line->maxThroughputUp() / (1024*8));
}


void LCMiniDisplay::updateClientStatus(int clientStatus)
{
	//qDebug("LCMiniDisplay::updateClientStatus()");
	bClientIsOnline = (clientStatus == CLT_ONLINE);
	updateVisibility();
}


void LCMiniDisplay::updateLineStatus(int lineStatus)
{
  	//qDebug("LCMiniDisplay::updateLineStatus()");
	switch(lineStatus)
	{
	case CST_DOWN:
	case CST_CLOSE_ERROR:
		bLineIsUp = false;
      resetView();
      break;

	case CST_DIALING:
	case CST_UP_SERVER:
	case CST_UP_USER:
	case CST_CLOSING:
	case CST_LOCKED_UP_CLT:
	case CST_LOCKED_DOWN:
	case CST_LOCKED_UP_TIMED:
		bLineIsUp = true;
		break;
	}
	updateVisibility();
}


void LCMiniDisplay::resetView()
{
	//qDebug("LCMiniDisplay::resetView()");
  	LcpLine l;
	l.setMaxThroughputUp(1000000);
	l.setMaxThroughputDown(1000000);
	updateSelectedLine(&l);

   t_lcp3_info_tput tmp;
   tmp.client = tmp.line = 0;
   tmp.dnl = tmp.upl = tmp.dns = tmp.ups = tmp.dnt = tmp.upt= 0;
   updateLineInfo(tmp, 0, 0);
}


void LCMiniDisplay::updateVisibility()
{
	//qDebug("LCMiniDisplay::updateVisibility()");
	switch (mShowMode)
	{
		case LCSettings::Never:
			hide();
			break;
		case LCSettings::Always:
			show();
			break;
		case LCSettings::ServerOnline:
			setVisible(bLineIsUp);
			break;
		case LCSettings::ClientAndServerOnline:
			setVisible(bLineIsUp && bClientIsOnline);
			break;
	}
}


void LCMiniDisplay::loadSettings()
{
	//qDebug("LCMiniDisplay::loadSettings()");
	LCSettings *s = LCSettings::self();
	setWindowOpacity(qBound(0.0, s->getMiniDispOpacity() / 100.0, 1.0));
	mShowMode = s->getMiniDispShowMode();
	lbLineTime->setVisible(s->getMiniDispShowLineTime());
   resize(qBound(64, s->getMiniDispWidth(), 800), minimumHeight());
	updateVisibility();
	//qDebug("requested width is %d, new width is now %d", s->getMiniDispWidth(), width());
}

