/***************************************************************************
                          misc.cpp  -  description
                             -------------------
    begin                : Fri Jun 8 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qcoreapplication.h>
#include <qdatetime.h>
#include <qstringlist.h>

// App includes
#include "misc.h"
#include "lcp3.h"


QString LCPIO::formatSize(unsigned int size)
{
	QString ret;
	if (size > 1024 * 1024 * 1024) // gigabyte
		ret.sprintf("%.2f GB", (double)size / (1024 * 1024 * 1024) );
	else if (size > 1024 * 1024) // megabyte
		ret.sprintf("%.2f MB", (double)size / (1024 * 1024) );
	else if (size > 1024) // kilobyte
		ret.sprintf("%.2f KB", (double)size / 1024);
	else // byte
		ret.sprintf("%.2f B", (double)size);

	return ret;
}


QString LCPIO::formatTime(unsigned int secs)
{
	unsigned int h, m, s;

	h = secs / 3600;
	m = (secs % 3600) / 60;
	s = (secs % 3600) % 60;

	QString time;
	//TODO i18n?
	time.sprintf("%02d:%02d:%02d", h, m, s);
	return time;
}


QString LCPIO::formatDate(QString year, QString mon, QString day)
{
	QStringList months;
	//TODO i18n?
	months << "Jan" << "Feb" << "Mar" << "Apr" << "May" << "Jun" << "Jul" <<
		"Aug" << "Sep" << "Oct" << "Nov" << "Dec";
	int m = months.indexOf(mon) + 1;

	QDate date(year.toInt(), m, day.toInt());
	return date.toString(Qt::LocalDate);
}


QString LCPIO::clientStatusToString(int status)
{
	switch (status)
	{
	case CLT_DISCON:
	case CLT_NOAUTH:
      return QCoreApplication::translate("LCPIO", "Disconnected", "client status");

	case CLT_OFFLINE:
      return QCoreApplication::translate("LCPIO", "Offline", "client status");

	case CLT_DIALING:
      return QCoreApplication::translate("LCPIO", "Dialing...", "client status");

	case CLT_ONLINE:
      return QCoreApplication::translate("LCPIO", "Online", "client status");

	case CLT_CLOSING:
      return QCoreApplication::translate("LCPIO", "Closing...", "client status");

	case CLT_ABORTING:
      return QCoreApplication::translate("LCPIO", "Aborting...", "client status");
	}
	return QString::null;
}


QString LCPIO::lineStatusToString(int status)
{
	switch (status)
	{
	case CST_DOWN:
		return QCoreApplication::translate("LCPIO", "Offline", "line status");

	case CST_DIALING:
		return QCoreApplication::translate("LCPIO", "Dialing...", "line status");

	case CST_UP_SERVER:
		return QCoreApplication::translate("LCPIO", "Online", "line status");

	case CST_UP_USER:
		return QCoreApplication::translate("LCPIO", "Online (User)", "line status");

	case CST_CLOSING:
		return QCoreApplication::translate("LCPIO", "Closing...", "line status");

	case CST_CLOSE_ERROR:
		return QCoreApplication::translate("LCPIO", "Error!", "line status");

	case CST_LOCKED_UP_CLT:
		return QCoreApplication::translate("LCPIO", "Online (Locked)", "line status");

	case CST_LOCKED_DOWN:
		return QCoreApplication::translate("LCPIO", "Offline (Locked)", "line status");

	case CST_LOCKED_UP_TIMED:
		return QCoreApplication::translate("LCPIO", "Online (Timelocked)", "line status");

	}
	return QString::null;
}

