/***************************************************************************
                          misc.h  -  description
                             -------------------
    begin                : Fri Jun 8 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MISC_H
#define MISC_H

// Qt includes
#include <qstring.h>

namespace LCPIO
{
	enum LogicType
	{
		AND = 0,
		OR = 1
	};

	enum ModeType
	{
		Normal = 0,
		Activity = 1
	};

	class OfflineInfo
	{
	public:
		bool autoOfflineActive;
		bool timeActive;
		int time;
		bool uploadActive;
		uint upload;
		bool downloadActive;
		uint download;
		bool maxUploadActive;
		uint maxUpload;
		bool maxDownloadActive;
		uint maxDownload;
		LogicType logic;
		ModeType mode;
	};

	enum CostType
	{
		Time = 0,
		Total = 1
	};

	class CostInfo
	{
	public:
		bool costCalcActive;
		double cost;
		double connectionFee;
		CostType costType;
	};

	//! format size from bytes to 12,34 [KB, MB, GB]
	//! @param size size in bytes to format
	QString formatSize(unsigned int size);

	//! format time from seconds to XX:XX:XX
	//! @param secs seconds to format
	QString formatTime(unsigned int secs);

	//! format date from strings to a localized string
	//! @param year the year
	//! @param mon the month
	//! @param day the day
	QString formatDate(QString year, QString mon, QString day);

	// @return Translated text for any of the CLT_* constants
	QString clientStatusToString(int status);
	
	// @return Translated text for any of the CST_* constants
	QString lineStatusToString(int status);
}

#endif //MISC_H
