/***************************************************************************
                          sendqpost.cpp  -  description
                             -------------------
    begin                : Mon Feb 12 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Standard headers
#include <stdlib.h>
//#include <unistd.h>
#include <string.h>
#include <stdio.h>

// Qt headers
#include <qtcpsocket.h>

// App headers
#include "sendqpost.h"
#include "lcp3.h"

SendQPost::SendQPost(int cmd, void* info, int infoSize)
{
	this->cmd = cmd;
	this->info = info;
	this->infoSize = (info ? infoSize : 0);
	this->prevSend = 0;
	this->sendCount = 0;
}

bool SendQPost::send(QTcpSocket* socket, int lineId)
{
	bool success = false;
	// Size of packet to send
	qint64 packetSize;
	// Buffer for packet creation
	char* buffer = (char*) malloc(get_lcp3_pack_size(cmd));
	// Pointer to the cmd part of buffer
	struct t_lcp3_cmd* bufferCmd = (struct t_lcp3_cmd*)buffer;
	// Pointer to the info part of buffer
	char* bufferInfo = ((char*)buffer) + sizeof(struct t_lcp3_cmd);

	bufferCmd->magic = LCP3_MAGICNR;
	bufferCmd->version = LCP3_PROTO_VERSION;
	bufferCmd->cmd = cmd;
	bufferCmd->info_is_valid = 1;
	bufferCmd->line_id = lineId;
	bufferCmd->line_stat = 0;
	bufferCmd->line_channels = 0;

	// Add info part if it exists
	if(info && infoSize > 0)
		memcpy(bufferInfo, info, infoSize);

	packetSize = sizeof(struct t_lcp3_cmd) + infoSize;

	//qDebug("Sending cmd %d for line %d with infosize %d, final size: %d", cmd, lineId, infoSize, packetSize);

	qint64 bytesWritten = socket->write(buffer, packetSize);
	success = (bytesWritten == packetSize);

	if (bytesWritten == -1)
		qDebug("qlinecontrol, error while writing to socket");
	else if (bytesWritten != packetSize)
		qDebug("qlinecontrol, could not write whole packet to socket");

	prevSend = time(NULL);
	sendCount++;
	free(buffer);

	return success;
}

bool SendQPost::isTimedOut() const
{
	return ((prevSend < (time(NULL) - LCP3_TMO_ACK)) && sendCount);
}

bool SendQPost::noTryLeft() const
{
	return (sendCount >= LCP3_TMO_CNT);
}

int SendQPost::getCmd() const
{
	return cmd;
}

