/***************************************************************************
                          sendqpost.h  -  description
                             -------------------
    begin                : Mon Feb 12 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SENDQPOST_H
#define SENDQPOST_H

// Standard headers
#include <time.h>

class QTcpSocket;

//! Post in the send queue
//! @author Peter Simonsson
class SendQPost
{
public:
	//! Constructor
	//! @param cmd LCP3 command
	//! @param info info struct for cmd
	//! @param infoSize size of info struct
	SendQPost(int cmd, void* info = NULL, int infoSize = 0);

	//! Send post to server
	//! @param socket the socket to write to
	bool send(QTcpSocket* socket, int lineId = 0);

	//! @return true if this command should be resent
	bool isTimedOut() const;

	//! @return true if this command should be removed from the send-queue
	bool noTryLeft() const;

	//! @return value of cmd
	int getCmd() const;
private:
	//! LCP3 command
	int cmd;
	
	//! Info struct for cmd
	void* info;

	//! Size of info
	int infoSize;

	//! Prev time cmd was sent (without success?)
	time_t prevSend;
	
	//! Number of times this packet has been sent
	unsigned int sendCount;
};

#endif
