/***************************************************************************
                               client settings
                             -------------------
    begin                : Thu Dec 01 18:08 CET 2005
    copyright            : (C) 2005 by Stefan Gehn
    email                : metz \AT\ gehn \DOT\ net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lcsettings.h"

#include <qapplication.h>
#include <qmainwindow.h>
#include <qdesktopwidget.h>
#include <qsettings.h>
#include <qdebug.h>

LCSettings* LCSettings::sSelf = 0;

LCSettings *LCSettings::self()
{
	if (!sSelf)
		sSelf = new LCSettings(qApp);
	return sSelf;
}

LCSettings::LCSettings(QObject *parent) : QObject(parent)
{
	qDebug("LCSettings::LCSettings()");
	s = new QSettings(QSettings::IniFormat, QSettings::UserScope,
		qApp->organizationName(), qApp->applicationName(), this);
}

LCSettings::~LCSettings()
{
	qDebug("LCSettings::~LCSettings()");
	s->sync();
	delete s;
}

void LCSettings::sync()
{
	qDebug("LCSettings::sync()");
	s->sync();
	emit settingsChanged();
}


QString LCSettings::getServerHost() const { return s->value("ServerHost").toString(); }
void LCSettings::setServerHost(const QString &val) { s->setValue("ServerHost", val); }

int LCSettings::getServerPort() const { return s->value("ServerPort", 16007).toInt(); }
void LCSettings::setServerPort(int val) { s->setValue("ServerPort", val); }

bool LCSettings::getUseAuth() const { return s->value("UseAuth", false).toBool(); }
void LCSettings::setUseAuth(bool val) { s->setValue("UseAuth", val); }

QString LCSettings::getAuthName() const { return s->value("AuthName").toString(); }
void LCSettings::setAuthName(const QString &val) { s->setValue("AuthName", val); }

void cr(QByteArray &d)
{
	int ki = 0;
	int dl = d.size();
	QByteArray k;
	k.append(2);
	k.append("qlcK01");
	k.append(126);
	k.append(35);
	int kl = k.size();
	for (int i = 0; i < dl; ++i)
	{
		//qDebug("using key char: %c", k.at(ki));
		d[i] = d[i] ^ k[ki];
		ki = (ki+1) % kl;
	}
}

QString LCSettings::getAuthPass() const
{
	QByteArray crArr = s->value("AuthPass").toByteArray();
	cr(crArr);
	return QString::fromUtf8(crArr.data(), crArr.size());
}

void LCSettings::setAuthPass(const QString &val)
{
	QByteArray crArr = val.toUtf8();
	cr(crArr);
	s->setValue("AuthPass", crArr);
}

bool LCSettings::getStoreAuthPass() const { return s->value("StoreAuthPass", false).toBool(); }
void LCSettings::setStoreAuthPass(bool val) { s->setValue("StoreAuthPass", val); }


// Client behaviour settings
bool LCSettings::getAutoConnect() const { return s->value("AutoConnect", false).toBool(); }
void LCSettings::setAutoConnect(bool val) { s->setValue("AutoConnect", val); }

bool LCSettings::getAutoOnline() const { return s->value("AutoOnline", false).toBool(); }
void LCSettings::setAutoOnline(bool val) { s->setValue("AutoOnline", val); }


// Look and feel settings
bool LCSettings::getRememberWindowState() const { return s->value("RememberWindowState", true).toBool(); }
void LCSettings::setRememberWindowState(bool val) { s->setValue("RememberWindowState", val); }

bool LCSettings::getUseTray() const { return s->value("UseTray", true).toBool(); }
void LCSettings::setUseTray(bool val) { s->setValue("UseTray", val); }


// minidisplay
int LCSettings::getMiniDispOpacity() const { return s->value("MiniDispOpacity", 100).toInt(); }
void LCSettings::setMiniDispOpacity(int val) { s->setValue("MiniDispOpacity", val); }

LCSettings::ShowMode LCSettings::getMiniDispShowMode() const { return (LCSettings::ShowMode)s->value("MiniDispShowMode", ServerOnline).toInt(); }
void LCSettings::setMiniDispShowMode(ShowMode val) { s->setValue("MiniDispShowMode", val); }

int LCSettings::getMiniDispWidth() const { return s->value("MiniDispWidth", 220).toInt(); }
void LCSettings::setMiniDispWidth(int val) { s->setValue("MiniDispWidth", val); }

QPoint LCSettings::getMiniDispPos() const
{
	QPoint pos = s->value("MiniDispPos").toPoint();
	QRect deskRect = qApp->desktop()->screenGeometry(pos);
	if (deskRect.contains(pos))
		return pos;
	else
		qDebug("[LCSettings] invalid window-position in config, ignoring");

	return QPoint();
}
void LCSettings::setMiniDispPos(QPoint val) { s->setValue("MiniDispPos", val); }

bool LCSettings::getMiniDispShowLineTime() const { return s->value("MiniDispShowLineTime", true).toBool(); }
void LCSettings::setMiniDispShowLineTime(bool val) { s->setValue("MiniDispShowLineTime", val); }


// general mainwindow
void LCSettings::saveWindowSettings(const QMainWindow *w)
{
	//qDebug("LCSettings::saveWindowSettings()");
	s->beginGroup(w->objectName());
	s->setValue("pos", w->pos());
	s->setValue("size", w->size());
	s->setValue("dock", w->saveState());
	s->setValue("hidden", w->isHidden());
	s->setValue("minimized", w->isMinimized());
	s->endGroup();
}

void LCSettings::loadWindowSettings(QMainWindow *w)
{
	//qDebug("LCSettings::loadWindowSettings()");
	s->beginGroup(w->objectName());
	QPoint pos = s->value("pos").toPoint();
	QSize size = s->value("size").toSize();

	if (!pos.isNull() && size.isValid())
	{
		// position and size both have to be valid and
		// the window rect has to be within the desktop rect
		QRect deskRect = qApp->desktop()->screenGeometry(pos);
		if (deskRect.contains(QRect(pos, size)))
		{
			w->resize(size);
			w->move(pos);
		}
		else
			qDebug("[LCSettings] invalid window-position or -size in config, ignoring");
	}

	QVariant dockConfig = s->value("dock");
	if (!dockConfig.isNull())
		w->restoreState(dockConfig.toByteArray());

	if (s->value("minimized", false).toBool())
		w->showMinimized();

	if (s->value("hidden", false).toBool())
		w->hide();
	else
		w->show();

	s->endGroup();
}

