/***************************************************************************
                               settings dialog
                             -------------------
    begin                : Sat Dec 03 14:48 CET 2005
    copyright            : (C) 2005 by Stefan Gehn
    email                : metz \AT\ gehn \DOT\ net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "lcsettingsdlg.h"
#include "lcsettings.h"
#include "qtreelistwidget.h"

#include "ui_lcsrvsettings.h"
#include "ui_lcclisettings.h"

#include <qlocale.h>
#include <qdialog.h>
#include <qboxlayout.h>
#include <qlineedit.h>
#include <qlabel.h>

LCSettingsDlg::LCSettingsDlg(QWidget *parent) :
	QSDialog(parent,
		tr("QLineControl - Settings"),
		QSDialog::CancelButton | QSDialog::ApplyButton | QSDialog::OkButton,
		QSDialog::OkButton)
{
	qDebug("LCSettingsDlg::LCSettingsDlg()");
	
	connect(this, SIGNAL(cancelClicked()), this, SLOT(reject()));
	connect(this, SIGNAL(applyClicked()), this, SLOT(slotSaveSettings()));
	connect(this, SIGNAL(okClicked()), this, SLOT(slotOkClicked()));

	mWidget = new QTreeListWidget(this);
	setMainWidget(mWidget);

	// Create prefs-pages ------

	QFrame *f;
	f = mWidget->addPage(tr("Server"), tr("Server settings"));
	mSrv = new Ui::SrvSettings();
	mSrv->setupUi(f);

	f = mWidget->addPage(tr("Client"), tr("Client settings"));
	mCli = new Ui::CliSettings();
	mCli->setupUi(f);
	
	mCli->cbShowMode->addItem(tr("Never"), (int)LCSettings::Never);
	mCli->cbShowMode->addItem(tr("Always"), (int)LCSettings::Always);
	mCli->cbShowMode->addItem(tr("If server is online"), (int)LCSettings::ServerOnline);
	mCli->cbShowMode->addItem(tr("If client is online"), (int)LCSettings::ClientAndServerOnline);

	// Make the Qt4 sizing chaos a bit less annoying
	QSize hint = sizeHint();
	int defaultHeight = hint.height(); //qMax(hint.height()/2, minimumHeight());
	int defaultWidth = qMax(hint.width(), 320);
	resize(defaultWidth, defaultHeight);
}


LCSettingsDlg::~LCSettingsDlg()
{
  	qDebug("LCSettingsDlg::~LCSettingsDlg()");
}


int LCSettingsDlg::exec()
{
	slotLoadSettings();
	return QDialog::exec();
}


void LCSettingsDlg::slotOkClicked()
{
	slotSaveSettings();
	accept();
}


void LCSettingsDlg::slotLoadSettings()
{
	qDebug("LCSettingsDlg::slotLoadSettings()");
	LCSettings *s = LCSettings::self();
	
	mSrv->edHost->setText(s->getServerHost());
	mSrv->sbPort->setValue(s->getServerPort());
	bool auth = s->getUseAuth();
	mSrv->gbAuth->setChecked(auth);
	mSrv->edName->setText(auth ? s->getAuthName() : QString::null);
	mSrv->edPass->setText(auth ? s->getAuthPass() : QString::null);
	mSrv->chStorePass->setChecked(s->getStoreAuthPass());

	mCli->cbAutoConnect->setChecked(s->getAutoConnect());
	mCli->cbAutoOnline->setChecked(s->getAutoOnline());
	mCli->cbRememberWindowState->setChecked(s->getRememberWindowState());
	mCli->cbTray->setChecked(s->getUseTray());
	
	int idx = mCli->cbShowMode->findData(s->getMiniDispShowMode());
	mCli->cbShowMode->setCurrentIndex(idx != -1 ? idx : 2),
 	mCli->slWidth->setValue(s->getMiniDispWidth());
	mCli->slOpacity->setValue(s->getMiniDispOpacity());
	mCli->cbShowLineTime->setChecked(s->getMiniDispShowLineTime());
}


void LCSettingsDlg::slotSaveSettings()
{
	qDebug("LCSettingsDlg::slotSaveSettings()");
	LCSettings *s = LCSettings::self();

	s->setServerHost(mSrv->edHost->text());
	s->setServerPort(mSrv->sbPort->value());
	
	bool auth = mSrv->gbAuth->isChecked();
	s->setUseAuth(auth);
	s->setAuthName(auth ? mSrv->edName->text() : QString::null);
	s->setStoreAuthPass(mSrv->chStorePass->isChecked());
	bool savePass = auth && mSrv->chStorePass->isChecked();
	// purge password from config if either auth or password-saving is disabled
	s->setAuthPass(savePass ? mSrv->edPass->text() : QString::null);
	
	s->setAutoConnect(mCli->cbAutoConnect->isChecked());
	s->setAutoOnline(mCli->cbAutoOnline->isChecked());
	s->setRememberWindowState(mCli->cbRememberWindowState->isChecked());
	s->setUseTray(mCli->cbTray->isChecked());

	s->setMiniDispShowMode((LCSettings::ShowMode)mCli->cbShowMode->itemData(mCli->cbShowMode->currentIndex()).toInt());
	s->setMiniDispWidth(mCli->slWidth->value());
	s->setMiniDispOpacity(mCli->slOpacity->value());
	s->setMiniDispShowLineTime(mCli->cbShowLineTime->isChecked());

	s->sync();
}

