/***************************************************************************
                      dialog with standard buttons at the bottom
                             -------------------
    begin                : Sat Sep 03 19:32 CET 2005
    copyright            : (C) 2005 by Stefan Gehn
    email                : metz \AT\ gehn \DOT\ net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qsdialog.h"
#include <qgridlayout.h>
#include <qpushbutton.h>

QSDialog::QSDialog(QWidget *parent, const QString &title,
	QSDialog::Buttons buttons, QSDialog::Button defaultButton) : QDialog(parent)
{
	mMainWidget = 0L;
	setWindowTitle(title);
	mLayout = new QGridLayout(this);
	mLayout->addLayout(createButtons(buttons, defaultButton), 1, 0);
}


QLayout *QSDialog::createButtons(QSDialog::Buttons buttons, QSDialog::Button def) // private
{
	QPushButton *ok = 0L, *cancel = 0L, *apply = 0L, *help = 0L;
	QHBoxLayout *buttonLayout = new QHBoxLayout();

	// --- create wanted buttons ...

	if (buttons & OkButton)
	{
		ok = new QPushButton(tr("&Ok"), this);
		ok->setAutoDefault(false);
		ok->setDefault(def == OkButton);
		connect(ok, SIGNAL(clicked()), this, SIGNAL(okClicked()));
	}
	if (buttons & CancelButton)
	{
		cancel = new QPushButton(tr("&Cancel"), this);
		cancel->setAutoDefault(false);
		cancel->setDefault(def == CancelButton);
		connect(cancel, SIGNAL(clicked()), this, SIGNAL(cancelClicked()));
	}
	if (buttons & ApplyButton)
	{
		apply = new QPushButton(tr("&Apply"), this);
		apply->setAutoDefault(false);
		apply->setDefault(def == ApplyButton);
		connect(apply, SIGNAL(clicked()), this, SIGNAL(applyClicked()));
	}
	if (buttons & HelpButton)
	{
		help = new QPushButton(tr("&Help"), this);
		help->setAutoDefault(false);
		help->setDefault(def == HelpButton);
		connect(help, SIGNAL(clicked()), this, SIGNAL(helpClicked()));
	}

	// --- and now add them in a sane order into the layout...

	if (help)
		buttonLayout->addWidget(help);

	buttonLayout->addStretch();

#ifdef WIN32
	if (ok)
		buttonLayout->addWidget(ok);
	if (cancel)
		buttonLayout->addWidget(cancel);
	if (apply)
		buttonLayout->addWidget(apply);
#else
	if (ok)
		buttonLayout->addWidget(ok);
	if (apply)
		buttonLayout->addWidget(apply);
	if (cancel)
		buttonLayout->addWidget(cancel);
#endif
	return buttonLayout;
}


void QSDialog::setMainWidget(QWidget *widget)
{
	if (mMainWidget)
		mLayout->removeWidget(mMainWidget);

	mMainWidget = widget;

	if (mMainWidget)
		mLayout->addWidget(mMainWidget, 0, 0);
}
