/***************************************************************************
                              tree list widget
                             -------------------
    begin                : Sat Jul 30 19:32 CET 2005
    copyright            : (C) 2005 by Stefan Gehn
    email                : metz \AT\ gehn \DOT\ net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qtreelistwidget.h"

#include <qstackedwidget.h>
#include <qboxlayout.h>
#include <qtreewidget.h>
#include <qheaderview.h>
//#include <qsplitter.h>
#include <qlabel.h>

QTreeListWidget::QTreeListWidget(QWidget *parent) : QWidget(parent)
{
	//qDebug("QTreeListWidget::QTreeListWidget() begin");
	QHBoxLayout *topLayout = new QHBoxLayout(this);
	topLayout->setMargin(0);
	topLayout->setSpacing(0);

	//QSplitter *splitter = new QSplitter(this);
	//topLayout->addWidget(splitter);

	mTree = new QTreeWidget(this);
	{
		QSizePolicy sp(QSizePolicy::Preferred, QSizePolicy::Preferred);
		mTree->setSizePolicy(sp);
		mTree->setColumnCount(1);
		mTree->header()->hide();
		mTree->setSelectionMode(QAbstractItemView::SingleSelection);
		mTree->setRootIsDecorated(false);
		connect (mTree, SIGNAL(itemSelectionChanged()), this, SLOT(slotTreeSelectionChanged()));
	}

	QFrame *page = new QFrame(this);
	{
		QSizePolicy sp(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
		page->setSizePolicy(sp);
		QVBoxLayout *vboxLayout = new QVBoxLayout(page);
		vboxLayout->setMargin(0);

		mTitle = new QLabel("emptytitle", page);
		QFont fnt(mTitle->font());
		fnt.setBold(true);
		mTitle->setFont(fnt);

		QFrame *line = new QFrame(page);
		line->setFrameStyle(QFrame::HLine | QFrame::Sunken);

		mStack = new QStackedWidget(page);

		vboxLayout->addWidget(mTitle);
		vboxLayout->addWidget(line);
		vboxLayout->addWidget(mStack);
	}

	//splitter->addWidget(mTree);
	//splitter->addWidget(page);
	topLayout->addWidget(mTree);
	topLayout->addSpacing(8);
	topLayout->addWidget(page);
	//qDebug("QTreeListWidget::QTreeListWidget() end");
}


QSize QTreeListWidget::minimumSizeHint() const
{
	QSize s1(layout()->spacing(), layout()->spacing()*2);
	QSize s2(0,0);
	QSize s3(0,0);
	QSize s4(mStack->minimumSizeHint());

	//s1.rwidth() += style()->pixelMetric(QStyle::PM_SplitterWidth);
	s2 = mTree->minimumSize();

	s3 += mTitle->sizeHint();
	//s3.rheight() += mTitleSep->minimumSize().height();

	//
	// Select the tallest item. It has only effect in IconList mode
	//
	int h1 = s1.rheight() + s3.rheight() + s4.height();
	int h2 = QMAX( h1, s2.rheight() );

	return QSize(s1.width()+s2.width()+QMAX(s3.width(),s4.width()), h2);
}


QSize QTreeListWidget::sizeHint() const
{
  return minimumSizeHint();
}


/*QFrame *QTreeListWidget::addPage(const QStringList &tree, const QString &pageHeader, const QIcon &icon)
{
	qDebug("TODO");
	return 0;
}*/


QFrame *QTreeListWidget::addPage(const QString &pageName, const QString &pageHeader, const QIcon &icon)
{
	QFrame *page = new QFrame(mStack);
	// The caller of addPage() can delete the returned frame in order to delete the added page.
	connect(page, SIGNAL(destroyed(QObject *)), SLOT(slotPageDestroyed(QObject *)));
	int pageIndex = mStack->addWidget(page);

	// Add page into tree on the left
	QTreeWidgetItem *twi = new QTreeWidgetItem(mTree);
	twi->setText(0, pageName);
	if (!icon.isNull())
	   twi->setIcon(0, icon);
	// link newly added tree-item to pageindex
	mTreeItemToIndex[twi] = pageIndex;
	// store title for page
	mIndexToHeader[pageIndex] = pageHeader;
	
	//qDebug("QTreeListWidget::addPage(); name = '%s', pageIndex = %d.", pageName.toLatin1().data(), pageIndex);

	// first page added is shown and should be selected
	if (mStack->currentIndex() == pageIndex)
		mTree->setItemSelected(twi, true);

	return page;
}

void QTreeListWidget::removePage(QFrame *page)
{
	if (!page)
		return;
	int pageIndex = mStack->indexOf(page);
	//qDebug("QTreeListWidget::removePage(), pageIndex = %d", pageIndex);
	if (pageIndex == -1) // not found
		return;
	// Find tree-widget for page
	QMap<QTreeWidgetItem *, int>::Iterator it;
	for(it = mTreeItemToIndex.begin(); it != mTreeItemToIndex.end(); ++it)
	{
		if (it.value() == pageIndex)
		{
			delete it.key(); // remove tree-item
			mStack->removeWidget(page); // remove page from dialog
			mTreeItemToIndex.remove(it.key()); // remove tree-item -> page-index association
			break;
		}
	}
}


void QTreeListWidget::setCurrentPage(QFrame *page) // public
{
	//qDebug("QTreeListWidget::setCurrentPage()");
	int pageIndex = mStack->indexOf(page);
	if (pageIndex == -1)
	{
		qDebug("QTreeListWidget::setCurrentPage(); page not found!");
		return;
	}
	// Find tree-widget for page
	QMap<QTreeWidgetItem *, int>::Iterator it;
	for(it = mTreeItemToIndex.begin(); it != mTreeItemToIndex.end(); ++it)
	{
		if (it.value() == pageIndex)
		{
			mTree->setItemSelected(it.key(), true);
			// selection change will now call
			// slotTreeSelectionChanged() if needed
			break;
		}
	}
}


QFrame *QTreeListWidget::currentPage() // public
{
	return static_cast<QFrame *>(mStack->currentWidget());
}


void QTreeListWidget::slotPageDestroyed(QObject *o)
{
	//qDebug("QTreeListWidget::slotPageDestroyed()");
	QFrame *page = static_cast<QFrame *>(o);
	removePage(page);
}


void QTreeListWidget::slotTreeSelectionChanged()
{
	//qDebug("QTreeListWidget::slotTreeSelectionChanged()");
	QList<QTreeWidgetItem *> selected = mTree->selectedItems();
	if (selected.empty())
		return;

	QMap<QTreeWidgetItem *, int>::const_iterator it;
	it = mTreeItemToIndex.find(selected.first());
	if (it != mTreeItemToIndex.end())
	{
		int pageIndex = it.value();
		/*qDebug("QTreeListWidget::slotTreeSelectionChanged(), setting
		 new current widget with pageindex %d", pageIndex);*/
		mStack->setCurrentIndex(pageIndex);
		mTitle->setText(mIndexToHeader[pageIndex]);
	}
}

